__author__ = "Lukas Leufen"
__date__ = '2019-11-14'

import argparse

from mlair.workflows import DefaultWorkflow
from mlair.data_handler.data_handler_mixed_sampling import DataHandlerMixedSamplingWithClimateFirFilter
from mlair.model_modules.fully_connected_networks import BranchedInputFCN as NN
from mlair.configuration.defaults import DEFAULT_PLOT_LIST, DEFAULT_TRAIN_END


STATS = {'o3': 'dma8eu', 'relhum': 'average_values', 'temp': 'maximum', 'u': 'average_values',
         'v': 'average_values', 'no': 'dma8eu', 'no2': 'dma8eu', 'cloudcover': 'average_values',
         'pblheight': 'maximum'}

DATA_ORIGIN = {"no": "", "no2": "", "o3": "",
               "cloudcover": "REA", "pblheight": "REA", "relhum": "REA",
               "temp": "REA", "u": "REA", "v": "REA"}


def main(parser_args):
    # plots = remove_items(DEFAULT_PLOT_LIST, "PlotConditionalQuantiles")
    args = dict(
        sampling=("hourly", "daily"),  # T1A, T1D
        stations=["DEBW107", "DEBW013"],  # [:5],  # T1B, TODO: remove indexing for meaningful experiments
        network="UBA",  # T1B
        variables=["o3", "no", "no2", "cloudcover", "pblheight", "relhum", "temp", "u", "v"],  # T1B
        statistics_per_var=STATS,  # T1A, T1D, T1F

        data_origin=DATA_ORIGIN,
        #         data_handler=DataHandlerMixedSampling,
        #        window_history_offset=16,
        #         window_history_size=6 * 24 + 16,  # T1D
        data_handler=DataHandlerMixedSamplingWithClimateFirFilter,
        filter_cutoff_period=[21],
        filter_order=[42],
        filter_window_type=("kaiser", 5),
        filter_add_unfiltered=True,
        apriori_sel_opts=slice(DEFAULT_TRAIN_END),
        apriori_type="residuum_stats",
        apriori_diurnal=True,
        use_filter_branches=True,

        window_history_size=2 * 24 + 16,
        window_history_offset=16,  # T1F
        window_lead_time=4,  # T1D
        target_var="o3",  # T1D
        interpolation_limit=(24, 2),  # T1F
        transformation={
            "o3": {"method": "log"},
            "no": {"method": "log"},
            "no2": {"method": "log"},
            "cloudcover": {"method": "min_max", "feature_range": [-1, 1]},
            "pblheight": {"method": "log"},
            "relhum": {"method": "min_max", "feature_range": [-1, 1]},
            "temp": {"method": "standardise"},
            "u": {"method": "standardise"},
            "v": {"method": "standardise"}, },  # T1F, also apply same target transformation

        start="2006-01-01",
        train_start="2006-01-01",
        end="2011-12-31",
        test_end="2011-12-31",

        train_model=False, create_new_model=True,
        epochs=1,
        model=NN,
        evaluate_bootstraps=False,
        bootstrap_type=["singleinput", "branch", "variable"],
        bootstrap_method=["shuffle", "zero_mean"],
        plot_list=DEFAULT_PLOT_LIST,
        # competitors=["IntelliO3-ts-v1", "MFCN_64_32_16_climFIR", "FCN_1449_512_32_4"],  # "FCN_1449_16_8_4",],
        lazy_preprocessing=True,
        use_multiprocessing=True,
        use_multiprocessing_on_debug=False,
        overwrite_local_data=False,
        overwrite_lazy_data=False,
        max_number_multiprocessing=2,
        **parser_args.__dict__
    )
    print(parser_args.__dict__)
    workflow = DefaultWorkflow(**args, start_script=__file__)
    workflow.run()


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('--experiment_date', metavar='--exp_date', type=str, default=None,
                        help="set experiment date as string")
    args = parser.parse_args(["--experiment_date", "testrun"])
    main(args)
